<?php

namespace FM\Service;

use FM\Factory\Fm00004000TarefasFactory;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;
use Doctrine\ORM\EntityManager;
use FM\Service\FuncionariosHierarqService;
use FM\Service\FindService;
use FM\Service\Fm00010000ParametrizacaoGeralService;

class Fm00004000TarefasService
{

    protected $TarefasFactory;
    protected $Environment;
    protected $newCode;
    protected $FuncionariosHierarqService;
    protected $FindService;
    protected $connection;
    protected $parametrizacoes;

    public function __construct(EntityManager $entityManager, Fm00004000TarefasFactory $TarefasFactory, FuncionariosHierarqService $FuncionariosHierarqService, FindService $FindService, Environment $Environment, NewCode $newCode, Fm00010000ParametrizacaoGeralService $Fm00010000ParametrizacaoGeralService)
    {
        $this->connection                   = $entityManager->getConnection();
        $this->TarefasFactory               = $TarefasFactory;
        $this->FuncionariosHierarqService   = $FuncionariosHierarqService;
        $this->FindService                  = $FindService;
        $this->Environment                  = $Environment;
        $this->newCode                      = $newCode;
        $this->parametrizacoes              = $Fm00010000ParametrizacaoGeralService;
    }

    // integração SM
    public function getContratos($parametros)
    {
        if (!empty($parametros['NRESTRUTURAM'])) {
            return $this->TarefasFactory->getContratos($parametros['NRESTRUTURAM']);
        }
    }

    public function getTarefasGrid()
    {
        $cdoperador = $this->Environment->getCdOperador();
        $listaOperadores = $this->FuncionariosHierarqService->montarHierarquiaOperadores($cdoperador);
        $params = array(
            "CDOPERADOR" => $cdoperador
        );

        return $this->TarefasFactory->getTarefasGrid($params, $listaOperadores);
    }

    public function getClientes()
    {
        return $this->TarefasFactory->getClientes();
    }

    public function getTipoLocalTarefa($parametros)
    {
        $arrayClientes = (is_array($parametros['CDCLIENTE'])) ? $this->FindService->formatarArray($parametros['CDCLIENTE']) : "'" . $parametros['CDCLIENTE'] . "'";
        return $this->TarefasFactory->getTipoLocalTarefa($arrayClientes);
    }

    public function getLocal($parametros)
    {
        $tipoLocal = ($parametros['UNICA'] == "S") ? $parametros['NRTIPOLOCALCLI'] : $this->FindService->formatarArray($parametros['NRTIPOLOCALCLI']);
        return $this->TarefasFactory->getLocal($tipoLocal, "'" . $parametros['CDCLIENTE'] . "'");
    }

    public function getChecklistTarefa($parametros)
    {

        $param = ($parametros['UNICA'] == "S") ? $parametros['NRTIPOLOCALCLI'] : $this->FindService->formatarArray($parametros['NRTIPOLOCALCLI']);
        return $this->TarefasFactory->getChecklistTarefa($param);
    }

    public function getItensChecklist($parametros)
    {
        return $this->TarefasFactory->getItensChecklist($parametros);
    }

    public function getEscopo()
    {
        return $this->TarefasFactory->getEscopo();
    }

    public function getTarefasSelect($parametros)
    {
        $param = implode(',', $parametros['NRATIVIDADECHK']);
        return $this->TarefasFactory->getTarefasSelect($param);
    }


    public function getCriterios($parametros)
    {

        if (!isset($parametros["MODAL"])) {
            $param = array(
                "NRTAREFACHK"    => implode(',', $parametros['NRTAREFACHK']),
                "NRATIVIDADECHK" => implode(',', $parametros['NRATIVIDADECHK']),
                "NRCHECKLIST"    => $parametros['NRCHECKLIST']
            );
        } else {
            $param = array(
                "NRTAREFACHK"    => $parametros['NRTAREFACHK'],
                "NRATIVIDADECHK" => $parametros['NRATIVIDADECHK'],
                "NRCHECKLIST"    => $parametros['NRCHECKLIST']
            );
        }

        return $this->TarefasFactory->getCriterios($param);
    }

    public function getFuncionario($parametros)
    {
        $local = '';
        $habilidade = '';

        $param = $this->parametrizacoes->getParametrizacoes();

        if (!empty($parametros['NRLOCALCLIENTE'])) {
            if (is_array($parametros['NRLOCALCLIENTE'])) {
                $local .= $this->FindService->formatarArray($parametros['NRLOCALCLIENTE']);
            } else {
                $local .= "'" . $parametros['NRLOCALCLIENTE'] . "'";
            }

            // if ($param[0]['IDUTILIZAHABILIDADE'] == 'S') {
            //     if (is_array($parametros['NRTAREFACHK'])) {
            //         $habilidade .= "AND NRTAREFACHK IN (". $this->FindService->formatarArray($parametros['NRTAREFACHK']) . ")";
            //     } else {
            //         $habilidade .= "AND NRTAREFACHK = '".$parametros['NRTAREFACHK']."'";
            //     }
            // }
            return $this->TarefasFactory->getFuncionario($local, $habilidade);
        }
    }

    public function getSupervisorTarefa($parametros)
    {
        if (!empty($parametros['NRLOCALCLIENTE'])) {
            $param = is_array($parametros['NRLOCALCLIENTE']) ? $this->FindService->formatarArray($parametros['NRLOCALCLIENTE']) : "'" . $parametros['NRLOCALCLIENTE'] . "'";
            return $this->TarefasFactory->getSupervisorTarefa($param);
        }
    }

    public function insertNovaTarefa($param)
    {
        $this->connection->beginTransaction();
        try {
            $NRORG =  $this->Environment->getNrOrg();

            if ($param['OBSERVACOES'] == 1) {
                $observacoesTarefa = "N";
                $observacoesVistoria = "N";
            } elseif ($param['OBSERVACOES'] == 2) {
                $observacoesTarefa = "S";
                $observacoesVistoria = "N";
            } elseif ($param['OBSERVACOES'] == 3) {
                $observacoesTarefa = "N";
                $observacoesVistoria = "S";
            } else {
                $observacoesTarefa = "S";
                $observacoesVistoria = "S";
            }

            foreach ($param["DADOSTAREFAS"] as $row) {
                if ($row["GERATAREFA"] == 'S') {
                    $nrchecklistagend = $this->newCode->getCodeByTableName("FCM_CHECKLISTAGEND", $NRORG, 1, 10, true);
                    $dataInicial = new \DateTime($row['DATAINICIAL']);
                    $dataFinal = new \DateTime($row['DATAFINAL']);
                    $params = array(
                        "NRCHECKLISTAGEND"      => $nrchecklistagend,
                        "NMCHECKLISTAGEND"      => $param['NOMETAREFA'],
                        "NRLOCALCLIENTE"        => $row['NRLOCALCLIENTE'],
                        "NRCHECKLIST"           => $param['NRCHECKLIST'],
                        "CDOPERADOR"            => $row['FUNCIONARIO'],
                        "IDSITUACAOAGEND"       => 1,
                        "DTHORAAGEND"           => $dataInicial->format('d/m/Y H:i'),
                        "DTHRFINALAGEND"        => $dataFinal->format('d/m/Y H:i'),
                        "CDOPERINCLUSAO"        => $this->Environment->getCdOperador(),
                        "IDOBRIGAOBSERVACAO"    => $observacoesTarefa,
                        "NRSEQCONTRATO"         => isset($row['NRSEQCONTRATO']) ? $row['NRSEQCONTRATO'] : null,
                        "IDTIPOMARCACAOTAREFA"  => $row['IDTIPOMARCACAOTAREFA'],
                        "IDTIPORECORRENCIA"     => $param['RECORRENCIA'],
                    );

                    $this->TarefasFactory->insertChecklist($params);

                    foreach ($param['NRTAREFACHK'] as $atividade) {
                        $nrtarefaagend = $this->newCode->getCodeByTableName("FCM_TAREFAAGEND", $NRORG, 1, 10, true);
                        $params = array(
                            "NRTAREFAAGEND" => $nrtarefaagend,
                            "NRCHECKLISTAGEND" => $nrchecklistagend,
                            "NRTAREFACHK" => $atividade,
                            "CDOPERADOR" => $row['FUNCIONARIO'],
                            "DTHORAAGEND" => $dataInicial->format('d/m/Y H:i'),
                            "DSOBSTAREFAAGEND" => null,
                            "CDOPERINCLUSAO" => $this->Environment->getCdOperador(),
                            "NRTAREFARECORRENTE" =>  isset($param["NRTAREFARECORRENTE"]) ? $param["NRTAREFARECORRENTE"] : null,
                        );

                        $this->TarefasFactory->insertTarefa($params);

                        foreach ($param['NRCRITERIOTAREFACHK'] as $criterios) {
                            //valida se o critério realmente pertence à atividade que está sendo inserida acima.
                            $paramsValida = array(
                                "NRTAREFACHK"      => $atividade,
                                "NRCRITERIOTAREFA" => $criterios['NRCRITERIOTAREFA']
                            );
                            $valida = $this->TarefasFactory->validaCriterio($paramsValida);
                            if ($valida > 0) {
                                $nrcriteriostaragend = $this->newCode->getCodeByTableName("FCM_CRITERIOTARAGEND", $NRORG, 1, 10, true);
                                $params = array(
                                    "NRCRITERIOTARAGEND" => $nrcriteriostaragend,
                                    "NMCRITERIOTARAGEND" => $criterios['DSCRITERIOTAREFA'],
                                    "NRTAREFAAGEND" => $nrtarefaagend,
                                    "NRCRITERIOTAREFACHK" => ($criterios['NRCRITERIOTAREFACHK'] === $criterios['DSCRITERIOTAREFA']) ? null : $criterios['NRCRITERIOTAREFACHK'],
                                    "OPERADOR" => $this->Environment->getCdOperador()
                                );

                                $this->TarefasFactory->insertCriterio($params);
                            }
                        }
                    }
                    if ($param["VISTORIA"] == 1) {
                        $nrgerenciamentochk = $this->newCode->getCodeByTableName("FCM_GERENCIAMENTOCHK", $NRORG, 1, 10, true);
                        $params = array(
                            "NRGERENCIAMENTOCHK"    => $nrgerenciamentochk,
                            "NMGERENCIAMENTOCHK"    => $param['NOMETAREFA'],
                            "NRLOCALCLIENTE"        => $row['NRLOCALCLIENTE'],
                            "NRCHECKLIST"           => $param['NRCHECKLIST'],
                            "CDOPERADOR"            => $row['ENCARREGADO'],
                            "DTHORAAGEND"           => $dataInicial->format('d/m/Y H:i'),
                            "DTHRFINALAGEND"        => $dataFinal->format('d/m/Y H:i'),
                            "OPERADOR"              => $this->Environment->getCdOperador(),
                            "NRCHECKLISTAGEND"      => $nrchecklistagend,
                            "IDOBRIGAOBSERVACAO"    => $observacoesVistoria,
                            "NRSEQCONTRATO"         => (isset($row['NRSEQCONTRATO'])) ? $row['NRSEQCONTRATO'] : null,
                        );

                        $this->TarefasFactory->insertChk($params);

                        foreach ($param['NRTAREFACHK'] as $atividade) {
                            $nrtarefaagendchk = $this->newCode->getCodeByTableName("FCM_TAREFAGERCHK", $NRORG, 1, 10, true);
                            $params = array(
                                "NRTAREFAGERCHK" => $nrtarefaagendchk,
                                "NRGERENCIAMENTOCHK" => $nrgerenciamentochk,
                                "NRTAREFACHK" => $atividade,
                                "CDOPERADOR" => $row['FUNCIONARIO'],
                                "DTHORATAREFAGERCHK" => $dataInicial->format('d/m/Y H:i'),
                                "DSOBSTAREFAGERCHK" => null,
                                "OPERADOR" => $this->Environment->getCdOperador()
                            );

                            $this->TarefasFactory->insertChkTarefa($params);

                            foreach ($param['NRCRITERIOTAREFACHK'] as $criterios) {
                                //valida se o critério realmente pertence à atividade que está sendo inserida acima.
                                $paramsValida = array(
                                    "NRTAREFACHK"      => $atividade,
                                    "NRCRITERIOTAREFA" => $criterios['NRCRITERIOTAREFA']
                                );

                                $valida = $this->TarefasFactory->validaCriterio($paramsValida);

                                if ($valida > 0) {
                                    $nrcriteriostaragerchk = $this->newCode->getCodeByTableName("FCM_CRITERIOTARGERCHK", $NRORG, 1, 10, true);
                                    $params = array(
                                        "NRCRITERIOTARGERCHK" => $nrcriteriostaragerchk,
                                        "NMCRITERIOTARGERCHK" => $criterios['DSCRITERIOTAREFA'],
                                        "NRTAREFAGERCHK" => $nrtarefaagendchk,
                                        "NRCRITERIOTAREFACHK" => ($criterios['NRCRITERIOTAREFACHK'] === $criterios['DSCRITERIOTAREFA']) ? null : $criterios['NRCRITERIOTAREFACHK'],
                                        "OPERADOR" => $this->Environment->getCdOperador()
                                    );

                                    $this->TarefasFactory->insertChkCriterio($params);
                                }
                            }
                        }
                    }
                }
            }


            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }


    public function inativarTarefas($parametros)
    {
        $this->connection->beginTransaction();
        try {
            $param = implode(',', $parametros["NRS"]);
            $this->TarefasFactory->deletarTarefas($param);
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function getTarefasSelectAll()
    {
        return $this->TarefasFactory->getTarefasSelectAll();
    }

    public function getDadosModalTarefas($params)
    {
        $paramAtividade = '';
        if (is_array($params['NRATIVIDADECHK']) && $params['NRATIVIDADECHK'] !== null) {
            $paramAtividade = $this->FindService->formatarArray($params['NRATIVIDADECHK']);
        } elseif ($params['NRATIVIDADECHK'] !== null) {
            $paramAtividade = $params['NRATIVIDADECHK'];
        }

        $escopo = $this->TarefasFactory->populaEscopo(array("NRCHECKLISTAGEND" => $params["NRCHECKLISTAGEND"]));

        $atividade = $this->TarefasFactory->getTarefasSelect($paramAtividade);

        $nrTarefachk = $this->TarefasFactory->getNrtarefachk(array(
            "NRATIVIDADECHK" => $params["NRATIVIDADECHK"],
            "NRCHECKLIST" => $params["NRCHECKLIST"]));

        return array(
            "ESCOPO"  => $this->formataValor($escopo),
            "ATIVIDADE" => $this->formataValor($atividade),
            "CRITERIOS" => (count($nrTarefachk) > 0) ? array($nrTarefachk[0]): null,
        );
    }

    public function formataValor($params)
    {
        if (isset($params['data'][0]["NMESCOPOSTAREFA"])) {
            return $params['data'][0]["NMESCOPOSTAREFA"];
        } elseif (isset($params['data'][0]["NMTAREFA"])) {
            return $params['data'][0]["NMTAREFA"];
        } else {
            $criterios = [];
            if (count($params['data']) > 0) {
                foreach ($params as $value) {
                    array_push($criterios, $value["NRCRITERIOTAREFACHK"]);
                }
            }
            return $criterios;
        }
    }
}
